@@ -17,12 +17,14 @@ from . import mutations
 from . import subscriptions
 from .dateHelpers import getUtcDate
 from .croniterHelpers import get_croniter
+from .redis import WorkerRedis
 
 class QueueType(Enum):
   EVENT= 'EVENT'
   FREQUENCY = 'FREQUENCY'
   ON_DEMAND = 'ON_DEMAND'
   SCHEDULE = 'SCHEDULE'
+  CONTINUOUS = 'CONTINUOUS'
 
 class JobStatus(Enum):
   ABORTED = 'ABORTED'
@@ -68,6 +70,9 @@ class ValiotWorker():
     self.jobConfigMode = JobConfigMode.KEEP
     self.context = None
     self.unsubJobCreated = None
+    self.useRedis = False
+    self.redis = WorkerRedis()
+    self.continuousJobs = []
     # logging attributes
     self.loggingStyle = LogStyle.PREFIX_FIRST_LINE
     self.stopLogging = False
@@ -92,6 +97,9 @@ class ValiotWorker():
   def setLoggingStyle(self, style):
     self.loggingStyle = style
 
+  def setUseRedis(self):
+    self.useRedis = True
+
   def job(
     self,
     name, 
@@ -787,7 +795,7 @@ class ValiotWorker():
       if (unsub):
         self.log(LogLevel.SUCCESS, f'REGISTERED LISTENER FOR EVENT QUEUE { queue["name"] }')
       else:
-        self.log(LogLevel.ERROR, f'ERROR REGISTERING LISTENER FOR EVENT QUEUE { queue["name"] }')
+        self.log(LogLevel.ERROR, f'ERROR REGISTERING LISTENER FOR EVENT QUEUE { queue["name"] }')      
 
   def registerJobUpdatesListener(self):
     def onJobUpdated(data):
@@ -808,7 +816,47 @@ class ValiotWorker():
       self.log( LogLevel.SUCCESS, f'REGISTERED LISTENER FOR JOB UPDATES' )
     else:
       self.log( LogLevel.ERROR, f'ERROR REGISTERING LISTENER FOR JOB UPDATES' )
+    
+  def registerContinuousJobs(self):
+    queuesByType = __.group_by(self.queues.values(), 'type')
+    shedeuleQueues = __.get(queuesByType, 'CONTINUOUS', default=[])
+    for shedeuleQueue in shedeuleQueues:
+      queue_name = __.get(shedeuleQueue, 'name', None)
+      if queue_name:
+        self.continuousJobs.append(queue_name)
+        queueLocks, errors = self.gql.query_one(queries.get_queueLocks, variables={
+                                               "workerCode": self.worker_code, "queueName": queue_name})                            
+        if errors:
+          self.log(LogLevel.ERROR, f'Error get_queueLocks:\n{errors}')
+          continue
+        if queueLocks:
+          self.log(LogLevel.WARNING, f'{queue_name} queue (CONTINUOUS type) has queueLocks')
+          variables = {
+            'id': __.get(queueLocks, 'id', None),
+            'active': False,
+            'endAt': getUtcDate(datetime.now()).strftime("%Y-%m-%dT%H:%M:%SZ")
+            }
+          _, errors = self.gql.mutate(mutations.update_lock, variables)
+          if errors:
+            self.log(LogLevel.ERROR, f'Error update_lock:\n{errors}')
+               
+    if self.continuousJobs:
+      self.log(LogLevel.SUCCESS, 'REGISTERED Continuous Queues:')
+      for continuousJob in self.continuousJobs:
+        self.log(LogLevel.SUCCESS, f"\t{continuousJob}")
 
+  def monitorContinuousJobs(self):
+    running_jobs_names = [__.get(running_job, 'job.queue.name', None)
+                         for _, running_job in self.runningJobs.items()]
+    for continuousJob in self.continuousJobs:
+      if not continuousJob in running_jobs_names and self.queues[continuousJob]['enabled']:
+        self.log( LogLevel.WARNING, f'Continuous Job {continuousJob} is not running' )
+        self.log( LogLevel.INFO, f'Creating Job {continuousJob} ...')
+        _, errors = self.gql.mutate(mutation=mutations.create_job,
+                        variables={'queueName': continuousJob,'worker': self.worker['code']})
+        if errors:
+          self.log(LogLevel.ERROR, f'Error create_job:\n{errors}')
+                
   def getProcessLoggerFunction(self, queue=None, job=None):
     """Logs to STDOUT formatted to help get helpful information
     
@@ -890,6 +938,9 @@ class ValiotWorker():
         self.processJobs()
       else:
         self.log(LogLevel.INFO, 'Listening to events...')
+      if self.useRedis:
+        self.redis.monitorRedisSubscriptions()
+      self.monitorContinuousJobs()
     except Exception as e:
       self.log(LogLevel.ERROR, f"Error in worker's event loop!")
       self.log(LogLevel.ERROR, f'e: {e}')
@@ -920,6 +971,10 @@ class ValiotWorker():
       self.registerJobCreatedSubscription()
     self.registerEventJobs()
     self.registerJobUpdatesListener()
+    if self.useRedis:
+      self.redis.setLog(self.log)
+      self.redis.initializeRedis()
+    self.registerContinuousJobs()
     while 1:
       try:
         self.eventLoop(interval=interval)
